' This example illustrates the use of the solveMCE add-in on the Smets-Wouters DSGE model

'************************************************************
' Section 1:  Workfile, model name, simulation range
	close @all 

'Workfile
  	wfcreate(wf=solveMCE_Smets_Wouters_model,page=sw) q 2000 2100

'useful strings
  	%mod			= "_s_w"			'Model name
  	%simstart 		= "2001q1"  	'Simulation range
 	%simend 		= "2050q4" 		'Simulation range
	%overridelist 	= "r_a "			'list with overridden variables 
	%alias 			= "_1"  			'alternative scenario



'************************************************************
' Section 2:  Model, coefficients, and data

	'specify the Smets-Wouters DSGE model
  	model {%mod}
  	{%mod}.append cons=c1*cons(-1)+(1-c1)*cons(+1)-c3*(r-pinf(+1))+c3*b
  	{%mod}.append inve=i1*inve(-1)+(1-i1)*inve(+1)+i2*pk+sinve
  	{%mod}.append pk=-(r-pinf(+1))+q1*pk(+1)+crk*rk(+1)/(1-ctou+crk)+eqs
  	{%mod}.append kp=(1-ctou)*kp(-1)+ctou*inve(-1)
  	{%mod}.append pinf=(cbeta*pinf(+1)+cindp*pinf(-1)+(1-cbeta*cprobp)*(1-cprobp)/cprobp*mc)/(1+cbeta*cindp)+epinf
  	{%mod}.append mc=calfa*rk+(1-calfa)*w-a
  	{%mod}.append w=cbeta*w1*w(+1)+w1*w(-1)+cbeta*w1*pinf(+1)-w2*pinf+w3*pinf(-1)-w4*(w-csigl*lab-csigma*(cons-chabb*cons(-1))/(1-chabb)+ls)+ew
  	{%mod}.append lab=-w+rk*((1+czcap)/czcap)+kp(-1)
  	{%mod}.append y=ccy*cons+ciy*inve+g
  	{%mod}.append zcap*czcap=rk
  	{%mod}.append  0*rk+y=cfc*(calfa*kp+calfa*(1/czcap)*rk+(1-calfa)*lab+a)
  	{%mod}.append r=crr*r(-1)+(1-crr)*(pinfbar+crpi*(pinf(-1)-pinfbar)+cry*(y-yf))+crdpi*(pinf-pinf(-1))+crdy*(y-yf-(y(-1)-yf(-1)))+em
  	{%mod}.append empl-empl(-1) = empl(+1)-empl+(lab-empl)*((1-cprobe)*(1-cprobe*cbeta)/(cprobe))
  	{%mod}.append  a=crhoa*a(-1)+ea
  	{%mod}.append ls=crhols*ls(-1)+els
  	{%mod}.append b=crhob*b(-1)+eb
  	{%mod}.append  sinve=crhoinve*sinve(-1)+einve
  	{%mod}.append  pinfbar=crhopb*pinfbar(-1)+epb
  	{%mod}.append g=crhog*g(-1)+eg
  	{%mod}.append 0*rf+cf=c1*cf(-1)+(1-c1)*cf(+1)-c3*rf+c3*b
  	{%mod}.append  invef=i1*invef(-1)+(1-i1)*invef(+1)+i2*pkf+sinve
  	{%mod}.append  pkf=-rf+q1*pkf(+1)+crk*rkf(+1)/(1-ctou+crk)
  	{%mod}.append kpf=(1-ctou)*kpf(-1)+ctou*invef(-1)
  	{%mod}.append yf=cfc*(calfa*kpf+calfa*(1/czcap)*rkf+(1-calfa)*labf+a)
  	{%mod}.append 0*cf+yf=ccy*cf+ciy*invef+g
  	{%mod}.append wf=csigl*labf+csigma*(cf-chabb*cf(-1))/(1-chabb)-ls
  	{%mod}.append  labf=-wf+rkf*((1+czcap)/czcap)+kpf(-1)
  	{%mod}.append emplf-emplf(-1) = emplf(+1)-emplf+(labf-emplf)*((1-cprobe)*(1-cprobe*cbeta)/(cprobe))
  	{%mod}.append -rkf*calfa=(1-calfa)*wf-a

	'set all variables to zero (all model variables are measured as deviation from steady state)
		%variables={%mod}.@varlist
		for %v {%variables}
			genr {%v}=0
		next
	
	'set parameter values
	cbeta=0.99
	ctou=0.025
	calfa=0.3
	czcap=0.1690
	cindp=0.469
	genr cindw=0.763
	genr clandaw=0.5
	cprobp=0.908
	genr cprobw=0.737
	csigl=2.4
	csigma=1.353
	chabb=0.573
	cfc=1.408
	genr csadjcost=6.771
	ccy=0.6
	ciy=0.22
	crdpi=0.14
	cry=0.099
	crdy=0.159
	crr=0.961
	crpi=1.684
	cprobe=0.5990
	crhols=0.889
	crhob=0.855
	crhoinve=0.927
	crhoa=0.823
	crhopb=0.924
	crhog=0.949
	
	crk=1/cbeta-1+ctou
	genr cg=1-ccy-ciy
	i1=1/(1+cbeta)
	i2=1/((1+cbeta)*csadjcost)
	q1=(1-ctou)/(crk+1-ctou)
	c1=chabb/(1+chabb)
	c3=(1-chabb)/(csigma*(1+chabb))
	w1=1/(1+cbeta)
	w2=(1+cbeta*cindw)/(1+cbeta)
	w3=cindw/(1+cbeta)
	w4=(1-cprobw)*(1-cbeta*cprobw)/(cprobw*(1+cbeta)*((1+clandaw)*csigl/clandaw+1))


'****************************************************************************************************************
' Section 3: Simulation

	'create add-factor that will be overridden
		smpl @all
			{%mod}.addassign @all 
			{%mod}.addinit(v=z) @all  
	
		smpl {%simstart} {%simstart}
			copy r_a r_a_{%alias}
			genr r_a{%alias} = r_a+1
	
		smpl {%simstart} {%simend}
	
	'solve model: solveMCE
		{%mod}.scenario(n, alias=%alias) %alias
		{%mod}.override {%overridelist}
	'	{%mod}.msg
	'	{%mod}.solve  'eviews-solver do not solve the model
		solveMCE  {%mod}


'****************************************************************************************************************
'Section 4: Graphs
  	series zero = 0
  	smpl %simstart %simstart + 60
  	graph gr1.line zero r{%alias} cons{%alias} y{%alias}
  	gr1.addtext(t,c,font(14)) "Positive interest rate shock"
  	show gr1


