'solution of forward looking model
'replicates simple one equation model in
'Taylor (1993) "Macroeconomic Policy in a World Economy,"
'Figure 1-1, p.14
close @all
wfcreate taylor1993 u 1 30

'paramters
	!beta 	= 0.5 			'set adjustment parameter
	!rho 	= 0.8 			'set AR(1) parameter for monetary shock
	!lag 	= 10			'set lag for anticipated shock
	!x0 	= !lag + 1	'axis label graph

'specify model
	model mod1
	mod1.append -pt + mt = -!beta*(pt(+1) - pt)

'create baseline (=actuals)
	series pt = 0 	'endogenous series
	series mt=0 	'exogenous shock

%case1 = "Unanticipated temporary shock"
	'shift exogenous variable
	copy mt mt_1
	smpl @first @first
	mt_1 = 1

	'create scenario and solve model
	mod1.scenario(n,alias="_1") %case1
	mod1.override mt
	smpl @first @last-1  'since pt(+1) needs to have a value, simulation must end in @last-1
	solveMCE(wfhide) mod1

	'plot response
	pt_1.displayname Unanticipated temporary shock 
	graph graph1.line pt_1

%case2 ="Unanticipated geometric shock"
	'shift exogenous variable
	copy mt mt_2
	smpl @first @first
	mt_2 = 1
	smpl @first+1 @last
	mt_2 = !rho*mt_2(-1)

	'create scenario and solve model
	mod1.scenario(n,alias="_2") %case2
	mod1.override mt
	smpl @first @last-1  'since pt(+1) needs to have a value, simulation must end in @last-1
	solveMCE(wfhide) mod1


	'plot response
	pt_2.displayname {%case2}
	graph graph2.line pt_2

%case3 = "Anticipated temporary shock"
	'shift exogenous variable
	copy mt mt_3
	smpl @first+!lag @first+!lag
	mt_3 = 1

	'create scenario and solve model
	mod1.scenario(n,alias="_3") %case3
	mod1.override mt
	smpl @first @last-1  'since pt(+1) needs to have a value, simulation must end in @last-1
	solveMCE(wfhide) mod1

	'plot response
	pt_3.displayname {%case3}
	graph graph3.line pt_3

%case4= "Anticipated geometric shock"
	'shift exogenous variable
	copy mt mt_4
	smpl @first+!lag @first+!lag
	mt_4 = 1
	smpl @first+!lag+1 @last
	mt_4 = !rho*mt_4(-1)

	'create scenario and solve model
	mod1.scenario(n,alias="_4") %case4
	mod1.override mt
	smpl @first @last-1  'since pt(+1) needs to have a value, simulation must end in @last-1
	solveMCE(wfhide) mod1

	'plot response
	pt_4.displayname {%case4}
	graph graph4.line pt_4
	graph4.draw(line,bottom,rgb(128,128,128)) !x0

'combine graphs
	freeze(g1) graph1 graph2 graph3 graph4
	g1.setelem(1) lpat(solid) symbol(circle)
	g1.scale(left) range(0,1)
	g1.addtext(t) Effect of shocks in forward looking models (Taylor Figure 1-1, p.14)
	g1.align(2,1.0,1.0)
	show g1


