' This example illustrates the use of the solveMCE add-in on a small macroeconomic model 
	'the example builds on the example provided as part of the mce_solve subroutine documentation
	'monetary policy shock - shifts add-factor in the interest rate rule

'************************************************************
' Section 1:  Workfile, model name, simulation range
	close @all 

' Workfile
  	%wfstart 		= "2000q1"
  	%wfend 			= "2100q4"
  	%mainpage 	= "sim"
  	wfcreate(wf=solveMCE,page={%mainpage}) q {%wfstart} {%wfend}

'useful strings
  	%mod			= "simple"		'Model name
  	%simstart 		= "2001q1"  	'Simulation range
 	%simend 		= "2025q4" 		'Simulation range
	%overridelist 	= "rate_a " 
	%alias 			= "_1"  


'************************************************************
' Section 2:  Model, coefficients, and data

' equations
  	model {%mod}
  	{%mod}.append pinf = cp(1) * pinf(-1) + (.98-cp(1))*pinf(1)+ cp(2) * ygap
  	{%mod}.append rate = cr(1)*rate(-1)+(1-cr(1))*(cr(2)*pinf + cr(3)*ygap)
  	{%mod}.append ygap = cy(1) * ygap(-1) + (.98-cy(1))*ygap(1) + cy(2) * (rate - pinf(1))

' coefficients
  	coef(2) cy
  	cy.fill .50, -.02
  	coef(2) cp
  	cp.fill .50, .02
  	coef(3) cr
  	cr.fill .75, 1.5, 0.5 

' set all data to zero
  	smpl @all
  	%vars = {%mod}.@varlist
  	for !i = 1 to @wcount(%vars)
    	%tmp = @word(%vars,!i)
    	series {%tmp} = 0
    	next

'************************************************************
' Section 3: Simulation

'create add-factor that will be overridden
	smpl @all
		{%mod}.addassign rate
		{%mod}.addinit(v=z) rate 

	smpl {%simstart} {%simstart}
		copy rate_a rate_a_{%alias}
		genr rate_a{%alias} = rate_a+1

	smpl {%simstart} {%simend}

'solve model: solveMCE
	{%mod}.scenario(n, alias=%alias) %alias
	{%mod}.override {%overridelist}
	solveMCE  {%mod}

'graphs
  	series zero = 0
  	smpl %simstart %simstart + 39
  	graph gr1.line zero rate{%alias} pinf{%alias} ygap{%alias}
  	gr1.addtext(t,c,font(14)) "Positive interest rate shock"
  	show gr1


