'Uncovered Interest rate Parity
'comparison of non-linear and linear log-approximation to UIP given a 1 pp interest rate increase abroad

close @all
rndseed 12345

'useful strings
	%simstart	= "2008"
	%simend	 = "2014"
	%alias 		= "_s1"	'alias for overridden variables
	%aliasb		= "_b1" 'alias for baseline scenario created in the scenario add-in
	%m			=	 "m"
	%smpl		= %simstart+ " " + %simend

'create workfile
	wfcreate(page=uip) q 2000 2020

'create actuals
	smpl @all
	genr nokeur =8+nrnd
	genr rutl = 2/400+nrnd/400
	genr rnok = 2/400
	genr e=nokeur

'specify model
	model {%m}
  	{%m}.append nokeur 			= nokeur(+1)*(1+rutl)/(1+rnok)  	'non-linear
  	{%m}.append log(e/e(+1)) 		= rutl-rnok								'linear log-approximation

'shift exogenous variable
	copy rutl rutl{%alias}
	smpl 2013 2013
	rutl{%alias} = rutl{%alias}+1/400

'solve for baseline
	{%m}.scenario(n,alias=%aliasb) "baseline "{%aliasb}
	copy rutl rutl{%aliasb}
	{%m}.override  rutl
	smpl {%simstart} {%simend}
	solveMCE(wfhide) {%m}

'initilise forward-looking variables based on baseline
	copy nokeur{%aliasb} nokeur{%alias}
	copy e{%aliasb} e{%alias}

'create scenario and solve model
	{%m}.scenario(n,alias=%alias) %case
	{%m}.override  rutl
	smpl {%simstart} {%simend}
	solveMCE(wfhide) {%m}

'plot response
	smpl {%simstart}-4 {%simend}+4
	graph graph.line() 100*(nokeur{%alias}/nokeur{%aliasb}-1) 100*(e{%alias}/e{%aliasb}-1)
	graph.setelem(1) legend("UIP")
	graph.setelem(2) legend("Log-linear approximation")
	graph.text(al) Exchange rate. Deviation from baseline in percent
	show graph


